﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;
//
namespace PerceptiveMCAPI.Methods
{
	public class campaignSchedule : Api_BaseExecute
	{
		private new campaignScheduleInput Input { get; set; }
		private campaignScheduleOutput Output { get; set; }
		// =============================================== constructors
		public campaignSchedule() { }
		// ---------
		public campaignSchedule( campaignScheduleInput input ) { Input = input; }
		// =============================================== Execute
		public campaignScheduleOutput Execute()
		{
			// empty constructor & Execute() is bad
			if ( Input == null )
				throw new ArgumentException( "Input class not provided" );
			// do the deed
			Output = Execute( Input );
			return Output;
		}
		// ---------------------------------------------
		public campaignScheduleOutput Execute( campaignScheduleInput input )
		{
			Input = input;
			Output = new campaignScheduleOutput( Input );
			ValidationIsOK = true;
			//-------------------------
			if ( Input.api_Validate )
			{
				Output.api_ValidatorMessages = ValidateInput.Validate( Input );
				ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
			}
			//------------------------- do it
			if ( ValidationIsOK )
			{
				ApiRun( Input, input.parms.apikey );  // execute API & format output
			}
			// ---
			return Output;
		}

		#region ================================================================= API Calls

		// ---------------------------------------------------------- XmlRpc
		protected override void ExecuteXmlRpc()
		{
			try
			{
				string schedule_time = apiHelper.FormatApiDateParm( Input.parms.schedule_time );
				string schedule_time_b = apiHelper.FormatApiDateParm( Input.parms.schedule_time_b );
				// ---------------------------------- 
				bool rpcResult = apiXmlRpc.api.campaignSchedule(
					Input.parms.apikey, Input.parms.cid, schedule_time, schedule_time_b );
				Output.Format( rpcResult );
			}
			catch ( XmlRpcFaultException fex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ---------------------------------------------------------- Serial Post
		protected override void ExecuteSerialPost()
		{
			try
			{
				Dictionary<string, string> dict = new Dictionary<string, string>();
				dict.Add( "apikey", Input.parms.apikey );
				dict.Add( "cid", Input.parms.cid );
				string sched = apiHelper.FormatApiDateParm( Input.parms.schedule_time );
				dict.Add( "schedule_time", sched );
				string sched_b = ( Input.parms.schedule_time_b == DateTime.MinValue )
						? "" : apiHelper.FormatApiDateParm( Input.parms.schedule_time_b );
				if ( sched_b != "" ) dict.Add( "schedule_time_b", sched_b );
				// -- do it
				string api_Request = "";
				string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
				Output.Format( api_Request, api_Response );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		#endregion
	}
}
