﻿using System;
using System.Collections.Generic;
using System.Web.Script.Serialization;
using System.Xml.Linq;
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Types
{
	public class campaignNotOpenedAIMOutput : Api_BaseOutput
	{
		public campaignNotOpenedAIMResults result { get; set; }
		public campaignNotOpenedAIMParms inputParms { get; private set; }
		// --------------------------------------------------------
		public campaignNotOpenedAIMOutput( campaignNotOpenedAIMInput input )
			: base( input )
		{
			result = new campaignNotOpenedAIMResults();
			inputParms = input.parms;
		}
		// --------------------------------------------------- tostring
		public override string ToString()
		{
			return string.Format( "Campaign Id: {0}, Not Opened Email Count: {1}",
				this.inputParms.cid, result.total );
		}

		#region ================================================================== format setup

		// ---- serial
		internal void Format( string api_Request, string api_Response )
		{
			this.api_Request = api_Request;
			this.api_Response = api_Response;
			// ----------------------------------
			if ( this.api_OutputType == EnumValues.OutputType.JSON )
				outputJSON();
			else if ( this.api_OutputType == EnumValues.OutputType.XML )
				outputXML();
		}

		// ---- xmlrpc
		internal void Format( x_campaignNotOpenedAIM rpcResult )
		{
			this.api_Request = EnumValues.NotSupplied;
			this.api_Response = EnumValues.NotSupplied;
			// ------------------------
			this.result = FormatResults( rpcResult );
		}

		#endregion

		#region ================================================================= Output

		// ---------------------------------------------------------- outputXML
		private void outputXML()
		{
			XElement xml = XElement.Parse( this.api_Response );
			// error check
			Api_Error error = apiSerial.GetXmlOutputError( xml );
			if ( error != null )
			{
				this.api_ErrorMessages.Add( error );
				return;
			}
			// -------------------------------------------- get values
			double dbl;
			try
			{
				double.TryParse( xml.Element( "total" ).Value, out dbl );
				this.result.total = dbl;
				//
				if ( xml.Element( "data" ).HasElements )
				{
					IEnumerable<XElement> unsubs = xml.Element( "data" ).Elements( "struct" );
					foreach ( XElement entry in unsubs )
					{
						campaignNotOpenedAIMResults.NotOpenedAIM noa = new campaignNotOpenedAIMResults.NotOpenedAIM();
						// ----------------------------------
						noa.email = entry.Value;
						this.result.data.Add( noa );
					}
				}
			}
			// -------------------------
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ---------------------------------------------------------- outputJSON
		private void outputJSON()
		{
			JavaScriptSerializer jss = new JavaScriptSerializer();

			try
			{
				if ( this.api_Response.Contains( EnumValues.JSONErrorString ) )
				{
					Api_Error err = jss.Deserialize<Api_Error>( this.api_Response );
					this.api_ErrorMessages.Add( err );
				}
				else
				{
					if ( string.IsNullOrEmpty( this.api_Response ) ||
						this.api_Response == "[]" ) return; // ignore empty set

					x_campaignNotOpenedAIM jsonResult = jss.Deserialize<x_campaignNotOpenedAIM>( this.api_Response );
					this.result = FormatResults( jsonResult );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ---------------------------------------------------------- FormatResults
		private campaignNotOpenedAIMResults FormatResults( x_campaignNotOpenedAIM x_results )
		{
			campaignNotOpenedAIMResults results = new campaignNotOpenedAIMResults();
			//
			results.total = x_results.total;

			foreach ( var item in x_results.data )
			{
				results.data.Add( new campaignNotOpenedAIMResults.NotOpenedAIM { email = item } );
			}
			//
			return results;
		}
		// ---------
		#endregion
	}
}
