﻿using System;
using System.Collections.Generic;
using System.Web.Script.Serialization;
using System.Xml.Linq;
//
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Types
{
	public class campaignBounceMessagesOutput : Api_BaseOutput
	{
		public campaignBounceMessagesResults result { get; set; }
		public campaignBounceMessagesParms inputParms { get; private set; }
		//
		public campaignBounceMessagesOutput( campaignBounceMessagesInput input )
			: base( input )
		{
			result = new campaignBounceMessagesResults();
			inputParms = input.parms;
		}
		// --------------------------------------------------- tostring
		public override string ToString()
		{
			return string.Format( "Bounce messages: {0}", this.result.total );
		}

		#region ================================================================== format setup

		// ---- serial
		internal void Format( string api_Request, string api_Response )
		{
			this.api_Request = api_Request;
			this.api_Response = api_Response;
			// ----------------------------------
			if ( this.api_OutputType == EnumValues.OutputType.JSON )
				outputJSON();
			else if ( this.api_OutputType == EnumValues.OutputType.XML )
				outputXML();
		}

		// ---- xmlrpc
		internal void Format( x_campaignBounceMessages rpcResult )
		{
			this.api_Request = EnumValues.NotSupplied;
			this.api_Response = EnumValues.NotSupplied;
			// ------------------------
			this.result = FormatResults( rpcResult );
		}

		#endregion

		#region ================================================================= Output

		// ---------------------------------------------------------- outputXML
		private void outputXML()
		{
			XElement xml = XElement.Parse( this.api_Response );
			// error check
			Api_Error error = apiSerial.GetXmlOutputError( xml );
			if ( error != null )
			{
				this.api_ErrorMessages.Add( error );
				return;
			}
			// -------------------------------------------- get values
			try
			{
				DateTime dt; int it;
				int.TryParse( xml.Element( "total" ).Value, out it );
				this.result.total = it;
				//
				IEnumerable<XElement> data = xml.Element( "data" ).Descendants( "struct" );
				foreach ( XElement entry in data )
				{
					this.result.data.Add( new campaignBounceMessagesResults.DataItem
						{
							date = DateTime.TryParse( entry.Element( "date" ).Value, out dt ) ? dt : dt,
							email = entry.Element( "email" ).Value,
							message = entry.Element( "message" ).Value
						} );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ---------------------------------------------------------- outputJSON
		private void outputJSON()
		{
			JavaScriptSerializer jss = new JavaScriptSerializer();
			try
			{
				if ( this.api_Response.Contains( EnumValues.JSONErrorString ) )
				{
					Api_Error err = jss.Deserialize<Api_Error>( this.api_Response );
					this.api_ErrorMessages.Add( err );
				}
				else
				{
					x_campaignBounceMessages jsonResult = jss.Deserialize<x_campaignBounceMessages>( this.api_Response );
					this.result = FormatResults( jsonResult );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// -------------------------------------------------------------------- FormatResults
		protected campaignBounceMessagesResults FormatResults( x_campaignBounceMessages x_results )
		{
			campaignBounceMessagesResults results = new campaignBounceMessagesResults();
			results.total = x_results.total;
			//
			DateTime dt = DateTime.MinValue;
			foreach ( var entry in x_results.data )
			{
				results.data.Add( new campaignBounceMessagesResults.DataItem
				{
					email = entry.email,
					message = entry.message,
					date = ( DateTime.TryParse( entry.date, out dt ) ) ? dt : dt
				} );
			}
			return results;
		}

		#endregion
	}
}
