﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Methods
{
	public class ecommOrders : Api_BaseExecute
	{
		private new ecommOrdersInput Input { get; set; }
		private ecommOrdersOutput Output { get; set; }
		// =============================================== constructors
		public ecommOrders() { }
		public ecommOrders( ecommOrdersInput input ) { this.Input = input; }

		#region =============================================== Mainline Execute

		public ecommOrdersOutput Execute()
		{
			// empty constructor & Execute() is bad
			if ( Input == null )
				throw new ArgumentException( "Input class not provided" );
			// do the deed
			Output = Execute( Input );
			return Output;
		}
		// ---------------------------------------------
		public ecommOrdersOutput Execute( ecommOrdersInput input )
		{
			Input = input;
			Output = new ecommOrdersOutput( Input );
			ValidationIsOK = true;
			//------------------------- validate it
			if ( Input.api_Validate )
			{
				Output.api_ValidatorMessages = ValidateInput.Validate( Input );
				ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
			}
			//------------------------- do it
			if ( ValidationIsOK )
			{
				ApiRun( Input, input.parms.apikey );  // execute API & format output
			}
			// ---
			return Output;
		}
		#endregion

		#region ================================================================= API Calls
		// ---------------------------------------------------------- XmlRpc
		protected override void ExecuteXmlRpc()
		{
			try
			{
				string since = apiHelper.FormatApiDateParm( Input.parms.since );
				x_ecommOrders rpcResult = apiXmlRpc.api.ecommOrders(
					Input.parms.apikey, Input.parms.start, Input.parms.limit, since );
				Output.Format( rpcResult );
			}
			catch ( XmlRpcFaultException fex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}
		// ---------------------------------------------------------- Serial Post
		protected override void ExecuteSerialPost()
		{
			try
			{
				Dictionary<string, string> dict = new Dictionary<string, string>();
				dict.Add( "apikey", Input.parms.apikey );
				if ( Input.parms.limit > 0 )
				{
					dict.Add( "start", Input.parms.start.ToString() );
					dict.Add( "limit", Input.parms.limit.ToString() );
				}
				if ( Input.parms.since > DateTime.MinValue )
				{
					string since = apiHelper.FormatApiDateParm( Input.parms.since );
					dict.Add( "since", since );
				}
				// -- do it
				string api_Request = "";
				string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
				Output.Format( api_Request, api_Response );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		#endregion
	}
}
