﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;

namespace PerceptiveMCAPI.Methods
{
	public class folderAdd : Api_BaseExecute
	{
		private new folderAddInput Input { get; set; }
		private folderAddOutput Output { get; set; }
		// =============================================== constructors
		public folderAdd() { }
		// ---------
		public folderAdd( folderAddInput input ) { Input = input; }
		// =============================================== Execute
		public folderAddOutput Execute()
		{
			// empty constructor & Execute() is bad
			if ( Input == null )
				throw new ArgumentException( "Input class not provided" );
			// do the deed
			Output = Execute( Input );
			return Output;
		}
		// ---------------------------------------------
		public folderAddOutput Execute( folderAddInput input )
		{
			Input = input;
			Output = new folderAddOutput( Input );
			ValidationIsOK = true;
			//-------------------------
			if ( Input.api_Validate )
			{
				Output.api_ValidatorMessages = ValidateInput.Validate( Input );
				ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
			}
			//------------------------- do it
			if ( ValidationIsOK )
			{
				ApiRun( Input, input.parms.apikey );  // execute API & format output
			}
			// ---
			return Output;
		}

		#region ================================================================= API Calls
		// ----------------------------------------------------------------------- xml-rpc
		protected override void ExecuteXmlRpc()
		{
			try
			{
				int rpcResult = apiXmlRpc.api.folderAdd( Input.parms.apikey, Input.parms.name, Input.parms.type.ToString() );
				Output.Format( rpcResult );
			}
			catch ( XmlRpcFaultException fex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}
		// ----------------------------------------------------------------------- serial
		protected override void ExecuteSerialPost()
		{
			try
			{
				Dictionary<string, string> dict = new Dictionary<string, string>();
				dict.Add( "apikey", Input.parms.apikey );
				dict.Add( "name", Input.parms.name );
				dict.Add( "type", Input.parms.type.ToString() );
				// -- do it
				string api_Request = "";
				string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
				Output.Format( api_Request, api_Response );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		#endregion
	}
}