﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;
//
namespace PerceptiveMCAPI.Methods
{
   public class listInterestGroupUpdate : Api_BaseExecute
   {
      private new listInterestGroupUpdateInput Input { get; set; }
      private listInterestGroupUpdateOutput Output { get; set; }
      // =============================================== constructors
      public listInterestGroupUpdate() { }
      // ---------
      public listInterestGroupUpdate( listInterestGroupUpdateInput input ) { Input = input; }
      // =============================================== Execute
      public listInterestGroupUpdateOutput Execute()
      {
         // empty constructor & Execute() is bad
         if ( Input == null )
            throw new ArgumentException( "Input class not provided" );
         // do the deed
         Output = Execute( Input );
         return Output;
      }
      // ---------------------------------------------
      public listInterestGroupUpdateOutput Execute( listInterestGroupUpdateInput input )
      {
         Input = input;
         Output = new listInterestGroupUpdateOutput( Input );
         ValidationIsOK = true;
         //-------------------------
         if ( Input.api_Validate )
         {
            Output.api_ValidatorMessages = ValidateInput.Validate( Input );
            ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
         }
         //------------------------- do it
         if ( ValidationIsOK )
         {
            ApiRun( Input, input.parms.apikey );  // execute API & format output
         }
         // ---
         return Output;
      }

      #region ================================================================= API Calls
      // ---------------------------------------------------------- XmlRpc
      protected override void ExecuteXmlRpc()
      {
         try
         {
            bool rpcResult = false;
            if ( Input.parms.grouping_id > 0 )
               rpcResult = apiXmlRpc.api.listInterestGroupUpdate(
                  Input.parms.apikey, Input.parms.id, Input.parms.old_name, Input.parms.new_name, Input.parms.grouping_id );
            else
               rpcResult = apiXmlRpc.api.listInterestGroupUpdate(
                  Input.parms.apikey, Input.parms.id, Input.parms.old_name, Input.parms.new_name );
            //
            Output.Format( rpcResult );
         }
         catch ( XmlRpcFaultException fex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }

      }

      // ---------------------------------------------------------- Serial Post
      protected override void ExecuteSerialPost()
      {
         try
         {
            Dictionary<string, string> dict = new Dictionary<string, string>();
            dict.Add( "apikey", Input.parms.apikey );
            dict.Add( "id", Input.parms.id );
            dict.Add( "old_name", Input.parms.old_name );
            dict.Add( "new_name", Input.parms.new_name );
            if ( Input.parms.grouping_id > 0 )
               dict.Add( "grouping_id", Input.parms.grouping_id.ToString() );
            // -- do it
            string api_Request = "";
            string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
            Output.Format( api_Request, api_Response );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }
      }

      #endregion
   }
}
