﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Methods
{
	public class listLocations : Api_BaseExecute
	{
		private new listLocationsInput Input { get; set; }
		private listLocationsOutput Output { get; set; }
		// =============================================== constructors
		public listLocations() { }
		// ---------
		public listLocations( listLocationsInput input ) { Input = input; }
		// =============================================== Execute
		public listLocationsOutput Execute()
		{
			// empty constructor & Execute() is bad
			if ( Input == null )
				throw new ArgumentException( "Input class not provided" );
			// do the deed
			Output = Execute( Input );
			return Output;
		}
		// ---------------------------------------------
		public listLocationsOutput Execute( listLocationsInput input )
		{
			Input = input;
			Output = new listLocationsOutput( Input );
			ValidationIsOK = true;
			//------------------------- validate it
			if ( Input.api_Validate )
			{
				Output.api_ValidatorMessages = ValidateInput.Validate( Input );
				ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
			}
			//------------------------- do it
			if ( ValidationIsOK )
			{
				ApiRun( Input, input.parms.apikey );  // execute API & format output
			}
			// ---
			return Output;
		}

		#region ================================================================= API Calls
		// ---------------------------------------------------------- XmlRpc
		protected override void ExecuteXmlRpc()
		{
			try
			{
				x_listLocations[] rpcResult = apiXmlRpc.api.listLocations( Input.parms.apikey, Input.parms.id );
				Output.Format( rpcResult );
			}
			catch ( XmlRpcFaultException fex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ----------------------------------------------------------------
		protected override void ExecuteSerialPost()
		{
			try
			{
				Dictionary<string, string> dict = new Dictionary<string, string>();
				dict.Add( "apikey", Input.parms.apikey );
				dict.Add( "id", Input.parms.id );
				// -- do it
				string api_Request = "";
				string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
				Output.Format( api_Request, api_Response );
			}
			catch ( Exception ex )
			{
				Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		#endregion
	}
}
