﻿using System;
using System.Collections.Generic;
using System.Web.Script.Serialization;
using System.Xml.Linq;
//
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Types
{
	public class listMembersOutput : Api_BaseOutput
	{
		public listMembersResults result { get; set; }
		public listMembersParms inputParms { get; private set; }
		// --------------------------------------------------- 
		public listMembersOutput( listMembersInput input )
			: base( input )
		{
			inputParms = input.parms;
			result = new listMembersResults();
		}
		// --------------------------------------------------- tostring
		public override string ToString()
		{
			return string.Format( "{0} Members on the list", result.total );
		}

		#region ================================================================== format setup
		// ---- serial
		internal void Format( string api_Request, string api_Response )
		{
			this.api_Request = api_Request;
			this.api_Response = api_Response;
			// ----------------------------------
			if ( this.api_OutputType == EnumValues.OutputType.JSON )
				outputJSON();
			else if ( this.api_OutputType == EnumValues.OutputType.XML )
				outputXML();
		}
		// --------------------------------------------------------------------
		// ---- xmlrpc
		internal void Format( x_listMembers rpcResult )
		{
			this.api_Request = EnumValues.NotSupplied;
			this.api_Response = EnumValues.NotSupplied;
			// ------------------------
			this.result = FormatResults( rpcResult );
		}
		#endregion
		#region ================================================================= Output
		// ---------------------------------------------------------- outputXML
		protected void outputXML()
		{
			XElement xml = XElement.Parse( this.api_Response );
			// error check
			Api_Error error = apiSerial.GetXmlOutputError( xml );
			if ( error != null )
			{
				this.api_ErrorMessages.Add( error );
				return;
			}
			// -------------------------------------------- get values
			try
			{
				int it; DateTime dt;
				//
				int.TryParse( xml.Element( "total" ).Value, out it );
				result.total = it;
				//
				IEnumerable<XElement> entries = xml.Element( "data" ).Elements( "struct" );
				foreach ( XElement entry in entries )
				{
					listMembersResults.data_item item = new listMembersResults.data_item();
					item.email = entry.Element( "email" ).Value;
					DateTime.TryParse( entry.Element( "timestamp" ).Value, out dt );
					item.timestamp = dt;
					if ( entry.Element( "reason" ) != null ) item.reason = entry.Element( "reason" ).Value;
					if ( entry.Element( "reason_text" ) != null ) item.reason_text = entry.Element( "reason_text" ).Value;
					//
					this.result.data.Add( item );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}
		// ---------------------------------------------------------- outputJSON
		protected void outputJSON()
		{
			JavaScriptSerializer jss = new JavaScriptSerializer();

			try
			{
				if ( this.api_Response.Contains( EnumValues.JSONErrorString ) )
				{
					Api_Error err = jss.Deserialize<Api_Error>( this.api_Response );
					this.api_ErrorMessages.Add( err );
				}
				else
				{
					x_listMembers jsonResult = jss.Deserialize<x_listMembers>( this.api_Response );
					this.result = FormatResults( jsonResult );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		#endregion
		// ---------------------------------------------------------- FormatResults
		protected listMembersResults FormatResults( x_listMembers x_results )
		{
			listMembersResults results = new listMembersResults();
			//
			results.total = x_results.total;
			//
			foreach ( var entry in x_results.data )
			{
				listMembersResults.data_item item = new listMembersResults.data_item();
				item.email = entry.email;
				DateTime dt; DateTime.TryParse( entry.timestamp, out dt );
				item.timestamp = dt;
				//
				item.reason = entry.reason ?? "";
				item.reason_text = entry.reason_text ?? "";
				//
				results.data.Add( item );
			}
			return results;
		}
	}
}
