﻿using System;
using System.Collections.Generic;
using System.Web.Script.Serialization;
using System.Xml.Linq;
//
using PerceptiveMCAPI.Types.Internal;
//
namespace PerceptiveMCAPI.Types
{
	public class listStaticSegmentMembersAddOutput : Api_BaseOutput
	{
		public listStaticSegmentMembersAddResults result { get; set; }
		public listStaticSegmentMembersAddParms inputParms { get; private set; }
		// --------------------------------------------------------
		public listStaticSegmentMembersAddOutput( listStaticSegmentMembersAddInput input )
			: base( input )
		{
			inputParms = input.parms;
			result = new listStaticSegmentMembersAddResults();
		}
		// --------------------------------------------------- tostring
		public override string ToString()
		{
			return string.Format( "Segment Members; added: {0}, errors: {1}", this.result.success, this.result.errors.Count );
		}

		#region ================================================================== format setup

		// ---- serial
		internal void Format( string api_Request, string api_Response )
		{
			this.api_Request = api_Request;
			this.api_Response = api_Response;
			// ----------------------------------
			if ( this.api_OutputType == EnumValues.OutputType.JSON )
				outputJSON();
			else if ( this.api_OutputType == EnumValues.OutputType.XML )
				outputXML();
		}

		// ---- xmlrpc
		internal void Format( x_listStaticSegmentMembersAdd rpcResult )
		{
			this.api_Request = EnumValues.NotSupplied;
			this.api_Response = EnumValues.NotSupplied;
			// ------------------------
			this.result = FormatResults( rpcResult );
		}

		#endregion

		#region ================================================================= Output

		// ---------------------------------------------------------- outputXML
		private void outputXML()
		{
			XElement xml = XElement.Parse( this.api_Response );
			// error check
			Api_Error error = apiSerial.GetXmlOutputError( xml );
			if ( error != null )
			{
				this.api_ErrorMessages.Add( error );
				return;
			}
			// -------------------------------------------- get values
			try
			{
				int it;
				listStaticSegmentMembersAddResults results = new listStaticSegmentMembersAddResults();

				int.TryParse( xml.Element( "success" ).Value, out it );
				results.success = it;
				if ( xml.Element( "errors" ).HasElements )
				{
					IEnumerable<XElement> errors = xml.Element( "errors" ).Elements( "struct" );
					foreach ( var elem in errors )
					{
						listStaticSegmentMembersAddError am_error = new listStaticSegmentMembersAddError();
						am_error.email = elem.Element( "email" ).Value;
						int.TryParse( elem.Element( "code" ).Value, out it );
						am_error.code = it;
						am_error.msg = elem.Element( "msg" ).Value;
						//
						results.errors.Add( am_error );
					}
				}
				this.result = results;
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}
		}

		// ---------------------------------------------------------- outputJSON
		private void outputJSON()
		{
			JavaScriptSerializer jss = new JavaScriptSerializer();

			try
			{
				if ( this.api_Response.Contains( EnumValues.JSONErrorString ) )
				{
					Api_Error err = jss.Deserialize<Api_Error>( this.api_Response );
					this.api_ErrorMessages.Add( err );
				}
				else
				{
					x_listStaticSegmentMembersAdd jsonResult = jss.Deserialize<x_listStaticSegmentMembersAdd>( this.api_Response );
					this.result = FormatResults( jsonResult );
				}
			}
			catch ( Exception ex )
			{
				this.api_ErrorMessages.Add( apiException.FormatError( ex ) );
			}

		}

		// ---------------------------------------------------------
		private listStaticSegmentMembersAddResults FormatResults( x_listStaticSegmentMembersAdd results_array )
		{
			listStaticSegmentMembersAddResults results = new listStaticSegmentMembersAddResults();
			//
			results.success = results_array.success;
			foreach ( x_listStaticSegmentMembersAddError error in results_array.errors )
			{
				results.errors.Add( new listStaticSegmentMembersAddError
				{
					email = error.email,
					code = error.code,
					msg = error.msg
				} );
			}
			//
			return results;
		}

		#endregion
	}
}
