﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;
//
namespace PerceptiveMCAPI.Methods
{
   public class listUnsubscribe : Api_BaseExecute
   {
      private new listUnsubscribeInput Input { get; set; }
      private listUnsubscribeOutput Output { get; set; }
      // =============================================== constructors
      public listUnsubscribe() { }
      // ---------
      public listUnsubscribe( listUnsubscribeInput input ) { Input = input; }
      // =============================================== Execute
      public listUnsubscribeOutput Execute()
      {
         // empty constructor & Execute() is bad
         if ( Input == null )
            throw new ArgumentException( "Input class not provided" );
         // do the deed
         Output = Execute( Input );
         return Output;
      }
      // ---------------------------------------------
      public listUnsubscribeOutput Execute( listUnsubscribeInput input )
      {
         Input = input;
         Output = new listUnsubscribeOutput( Input );
         ValidationIsOK = true;
         //------------------------- validate it
         if ( Input.api_Validate )
         {
            Output.api_ValidatorMessages = ValidateInput.Validate( Input );
            ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
         }
         //------------------------- do it
         if ( ValidationIsOK )
         {
            ApiRun( Input, input.parms.apikey );  // execute API & format output
         }
         // ---
         return Output;
      }

      #region ================================================================= API Calls
 
      // ---------------------------------------------------------- XmlRpc
      protected override void ExecuteXmlRpc()
      {
         try
         {
            bool rpcResult = apiXmlRpc.api.listUnsubscribe( Input.parms.apikey, Input.parms.id, Input.parms.email_address,
               Input.parms.delete_member, Input.parms.send_goodbye, Input.parms.send_notify );
            Output.Format( rpcResult );
         }
         catch ( XmlRpcFaultException fex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }
      }
      // ---------------------------------------------------------- Serial Post
      protected override void ExecuteSerialPost()
      {
         try
         {
            Dictionary<string, string> dict = new Dictionary<string, string>();
            dict.Add( "apikey", Input.parms.apikey );
            dict.Add( "id", Input.parms.id );
            dict.Add( "email_address", Input.parms.email_address );
            dict.Add( "delete_member", Input.parms.delete_member.ToString() );
            dict.Add( "send_goodbye", Input.parms.send_goodbye.ToString() );
            dict.Add( "send_notify", Input.parms.send_notify.ToString() );
            // -- do it
            string api_Request = "";
            string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
            Output.Format( api_Request, api_Response );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }
      }

      #endregion
   }
}
