﻿using System;
using System.Collections.Generic;
using CookComputing.XmlRpc;
//
using PerceptiveMCAPI.Types;

namespace PerceptiveMCAPI.Methods
{
   public class apikeyAdd : Api_BaseExecute
   {
      private new apikeyAddInput Input { get; set; }
      private apikeyAddOutput Output { get; set; }
      // =============================================== constructors
      public apikeyAdd() { }
      // ---------
      public apikeyAdd( apikeyAddInput input ) { Input = input; }
      // =============================================== Execute
      public apikeyAddOutput Execute()
      {
         // empty constructor & Execute() is bad
         if ( Input == null )
            throw new ArgumentException( "Input class not provided" );
         // do the deed
         Output = Execute( Input );
         return Output;
      }
      // ---------------------------------------------
      public apikeyAddOutput Execute( apikeyAddInput input )
      {
         Input = input;
         Output = new apikeyAddOutput( Input );
         ValidationIsOK = true;
         //-------------------------
         if ( Input.api_Validate )
         {
            Output.api_ValidatorMessages = ValidateInput.Validate( Input );
            ValidationIsOK = ValidateInput.IsOK( Output.api_ValidatorMessages );
         }
         //------------------------- do it
         if ( ValidationIsOK )
         {
            ApiRun( Input, input.parms.apikey );  // execute API & format output
         }
         // ---
         return Output;
      }

      #region ================================================================= API Calls
      // ----------------------------------------------------------------------- xml-rpc
      protected override void ExecuteXmlRpc()
      {
         try
         {
            string rpcResult = apiXmlRpc.api.apikeyAdd( Input.parms.username, Input.parms.password, Input.parms.apikey );
            Output.Format( rpcResult );
         }
         catch ( XmlRpcFaultException fex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( fex ) );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }
      }
      // ----------------------------------------------------------------------- serial
      protected override void ExecuteSerialPost()
      {
         try
         {
            Dictionary<string, string> dict = new Dictionary<string, string>();
            dict.Add( "username", Input.parms.username );
            dict.Add( "password", Input.parms.password );
            dict.Add( "apikey", Input.parms.apikey );
            // -- do it
            string api_Request = "";
            string api_Response = apiSerial.ProcessRequest( Input, dict, out api_Request );
            Output.Format( api_Request, api_Response );
         }
         catch ( Exception ex )
         {
            Output.api_ErrorMessages.Add( apiException.FormatError( ex ) );
         }
      }
      #endregion
   }
}